#ifndef RFBDISPATCH_H
#define RFBDISPATCH_H

#include <QObject>
#include <rfb/rfbclient.h>

class Uploader;

class Dispatcher : public QObject
{
    Q_OBJECT

public:
    explicit Dispatcher(Uploader *uploader, QObject *parent = Q_NULLPTR);
    ~Dispatcher();

    // libvnc hooks
    static rfbBool MallocFrameBuffer(rfbClient* client);
    static void GotCopyRect(rfbClient* client, const int src_x, const int src_y, const int w, const int h, const int dest_x, const int dest_y);
    static void GotFillRect(rfbClient* client, int x, int y, int w, int h, uint32_t colour);
    static void GotBitmap(rfbClient* client, const uint8_t* buffer, int x, int y, int w, int h);
    static rfbBool GotJpeg(rfbClient* client, const uint8_t* buffer, int length, int x, int y, int w, int h);
    static void FinishedFrameBufferUpdate(rfbClient* client);
    static char *GetPassword(rfbClient* client);

    // input events
    bool mouseEvent(const int x, const int y, const bool left, const bool middle, const bool right, const bool up, const bool down);

    // parameters of the VNC connection
    int bitsPerPixel();
private:
    rfbClient *m_client;
signals:
    // libvnc events
    void gotCopy(const int src_x, const int src_y, const int width, const int height, const int dest_x, const int dest_y);
    void gotFill(const int x, const int y, const int width, const int height, const float red, const float green, const float blue);
    void gotBitmap(const QImage &image, const quint32 x, const quint32 y, const quint32 width, const quint32 height);
    void finishedUpdate();
    void resizeFramebuffer(const QSize &size);
    void error();
public slots:
    // manage VNC connection
    void open(QString host, int port, const int width, const int height);
    void refresh();
    void decode();
    void terminate();
};

#endif // RFBDISPATCH_H
